from django.utils.decorators import decorator_from_middleware_with_args

from .middleware import CacheMiddleware


def cache_page(*args, **kwargs):
    cache_alias = kwargs.pop("cache", None)
    return decorator_from_middleware_with_args(CacheMiddleware)(
        cache_alias=cache_alias, *args, **kwargs
    )



from django.shortcuts import render
from django.conf import settings

from fancy_cache.memory import find_urls


def home(request):
    data = {
        "found": find_urls([]),
        "remember_all_urls_setting": getattr(
            settings, "FANCY_REMEMBER_ALL_URLS", False
        ),
        "remember_stats_all_urls_setting": getattr(
            settings, "FANCY_REMEMBER_STATS_ALL_URLS", False
        ),
    }
    return render(request, "fancy-cache/home.html", data)



from django.conf.urls import url
from . import views


urlpatterns = [
    url(r"^$", views.home, name="home"),
    # url(r'^2$', views.home2, name='home2'),
    # url(r'^3$', views.home3, name='home3'),
]



from django.test import TestCase
from nose.tools import eq_, ok_
from django.core.cache import cache
from django.core.management import call_command
from django.test import TestCase
from io import StringIO
from fancy_cache.middleware import REMEMBERED_URLS_KEY


class TestBaseCommand(TestCase):
    def setUp(self):
        self.urls = {
            "/page1.html": "key1",
            "/page2.html": "key2",
            "/page3.html?foo=bar": "key3",
            "/page3.html?foo=else": "key4",
        }
        for key, value in self.urls.items():
            cache.set(value, key)
        cache.set(REMEMBERED_URLS_KEY, self.urls, 5)

    def tearDown(self):
        cache.clear()

    def test_fancyurls_command(self):
        out = StringIO()
        call_command("fancy-urls", verbosity=3, stdout=out)
        self.assertIn("4 URLs cached", out.getvalue())

    def test_purge_command(self):
        out = StringIO()
        # Note: first call will show 4 URLs, so call again to confirm deletion
        call_command("fancy-urls", "--purge")
        call_command("fancy-urls", verbosity=3, stdout=out)
        self.assertIn("0 URLs cached", out.getvalue())



import os

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = "anything"

ALLOWED_HOSTS = ("testserver",)

DATABASES = {
    "default": {
        "NAME": ":memory:",
        "ENGINE": "django.db.backends.sqlite3",
    }
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "unique-snowflake",
    },
    "second_backend": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "unique-snowflake",
    },
}

INSTALLED_APPS = [
    "fancy_cache",
    "fancy_tests.tests",
    "django.contrib.auth",
    "django.contrib.contenttypes",
]


ROOT_URLCONF = "fancy_tests.tests.urls"


TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]



import unittest

from nose.tools import eq_, ok_
from django.core.cache import cache

from fancy_cache.middleware import REMEMBERED_URLS_KEY
from fancy_cache.memory import find_urls


class TestMemory(unittest.TestCase):
    def setUp(self):
        self.urls = {
            "/page1.html": "key1",
            "/page2.html": "key2",
            "/page3.html?foo=bar": "key3",
            "/page3.html?foo=else": "key4",
        }
        for key, value in self.urls.items():
            cache.set(value, key)
        cache.set(REMEMBERED_URLS_KEY, self.urls, 5)

    def tearDown(self):
        cache.clear()

    def test_find_all_urls(self):
        found = list(find_urls([]))
        eq_(len(found), 4)
        for key, value in self.urls.items():
            pair = (key, value, None)
            ok_(pair in found)

    def test_find_and_purge_all_urls(self):
        found = list(find_urls([], purge=True))
        eq_(len(found), 4)
        for key, value in self.urls.items():
            pair = (key, value, None)
            ok_(pair in found)
        found = list(find_urls([]))
        eq_(len(found), 0)

    def test_find_one_url(self):
        found = list(find_urls(["/page1.html"]))
        eq_(len(found), 1)
        ok_(("/page1.html", "key1", None) in found)

    def test_purge_one_url(self):
        ok_(cache.get("key1"))
        ok_("/page1.html" in cache.get(REMEMBERED_URLS_KEY))
        found = list(find_urls(["/page1.html"], purge=True))
        eq_(len(found), 1)
        ok_(("/page1.html", "key1", None) in found)

        ok_(not cache.get("key1"))
        ok_("/page1.html" not in cache.get(REMEMBERED_URLS_KEY))
        # find all the rest in there
        found = list(find_urls([]))
        eq_(len(found), 3)
        ok_(("/page1.html", "key1", None) not in found)

    def test_some_urls(self):
        found = list(find_urls(["/page2.html*"]))
        eq_(len(found), 1)
        ok_(("/page2.html", "key2", None) in found)

    def test_some_urls_double_star(self):
        found = list(find_urls(["/page*.html?*"]))
        eq_(len(found), 2)
        ok_(("/page3.html?foo=bar", "key3", None) in found)
        ok_(("/page3.html?foo=else", "key4", None) in found)



import uuid
from django.shortcuts import render
from fancy_cache import cache_page


def _view(request):
    random_string = uuid.uuid4().hex
    return render(request, "home.html", dict(random_string=random_string))


@cache_page(60)
def home(request):
    return _view(request)


def prefixer1(request):
    if request.META.get("AUTH_USER"):
        # disable
        return None
    return "a_key"


@cache_page(60, key_prefix=prefixer1)
def home2(request):
    return _view(request)


def post_processor1(response, request):
    assert "In your HTML" not in response.content.decode("utf8")
    response.content += ("In your HTML:%s" % uuid.uuid4().hex).encode("utf8")
    return response


@cache_page(60, key_prefix=prefixer1, post_process_response=post_processor1)
def home3(request):
    return _view(request)


@cache_page(
    60, key_prefix=prefixer1, post_process_response_always=post_processor1
)
def home4(request):
    return _view(request)


@cache_page(60, only_get_keys=["foo", "bar"])
def home5(request):
    return _view(request)


@cache_page(60, forget_get_keys=["bar"])
def home5bis(request):
    return _view(request)


@cache_page(60, remember_stats_all_urls=True, remember_all_urls=True)
def home6(request):
    return _view(request)


@cache_page(60, cache="second_backend")
def home7(request):
    return _view(request)



import unittest
import re
from nose.tools import eq_, ok_
from django.test.client import RequestFactory
from django.core.cache import cache, caches
from fancy_cache.memory import find_urls

from . import views


class TestViews(unittest.TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def tearDown(self):
        cache.clear()

    def test_render_home1(self):
        request = self.factory.get("/anything")

        response = views.home(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_1 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]

        # do it again
        response = views.home(request)
        eq_(response.status_code, 200)
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        eq_(random_string_1, random_string_2)

    def test_render_home2(self):
        authenticated = RequestFactory(AUTH_USER="peter")
        request = self.factory.get("/2")

        response = views.home2(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_1 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]

        # do it again
        response = views.home2(request)
        eq_(response.status_code, 200)
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        eq_(random_string_1, random_string_2)

        # do it again, but with a hint to disable cache
        request = authenticated.get("/2")
        response = views.home2(request)
        eq_(response.status_code, 200)
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        ok_(random_string_1 != random_string_2)

    def test_render_home3(self):
        request = self.factory.get("/anything")

        response = views.home3(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_1 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        ok_("In your HTML" in response.content.decode("utf8"))
        extra_random_1 = re.findall(
            "In your HTML:(\w+)", response.content.decode("utf8")
        )[0]

        response = views.home3(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        extra_random_2 = re.findall(
            "In your HTML:(\w+)", response.content.decode("utf8")
        )[0]
        ok_("In your HTML" in response.content.decode("utf8"))
        eq_(random_string_1, random_string_2)
        # the post_process_response is only called once
        eq_(extra_random_1, extra_random_2)

    def test_render_home3_no_cache(self):
        factory = RequestFactory(AUTH_USER="peter")
        request = factory.get("/3")

        response = views.home3(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        ok_("In your HTML" not in response.content.decode("utf8"))

    def test_render_home4(self):
        request = self.factory.get("/4")

        response = views.home4(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_1 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        ok_("In your HTML" in response.content.decode("utf8"))
        extra_random_1 = re.findall(
            "In your HTML:(\w+)", response.content.decode("utf8")
        )[0]

        response = views.home4(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        extra_random_2 = re.findall(
            "In your HTML:(\w+)", response.content.decode("utf8")
        )[0]
        ok_("In your HTML" in response.content.decode("utf8"))
        eq_(random_string_1, random_string_2)
        # the post_process_response is now called every time
        ok_(extra_random_1 != extra_random_2)

    def test_render_home5(self):
        request = self.factory.get("/4", {"foo": "bar"})
        response = views.home5(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_1 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]

        request = self.factory.get("/4", {"foo": "baz"})
        response = views.home5(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        ok_(random_string_1 != random_string_2)

        request = self.factory.get("/4", {"foo": "baz", "other": "junk"})
        response = views.home5(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_3 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        eq_(random_string_2, random_string_3)

    def test_render_home5bis(self):
        request = self.factory.get("/4", {"foo": "bar"})
        response = views.home5bis(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_1 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]

        request = self.factory.get("/4", {"foo": "baz"})
        response = views.home5bis(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        ok_(random_string_1 != random_string_2)

        request = self.factory.get("/4", {"foo": "baz", "bar": "foo"})
        response = views.home5bis(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_3 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        eq_(random_string_2, random_string_3)

    def test_remember_stats_all_urls(self):
        request = self.factory.get("/anything")
        response = views.home6(request)
        eq_(response.status_code, 200)

        # now ask the memory thing
        (match,) = find_urls(urls=["/anything"])
        eq_(match[0], "/anything")
        eq_(match[2]["hits"], 0)
        eq_(match[2]["misses"], 1)

        # second time
        response = views.home6(request)
        eq_(response.status_code, 200)
        (match,) = find_urls(urls=["/anything"])
        eq_(match[0], "/anything")
        eq_(match[2]["hits"], 1)
        eq_(match[2]["misses"], 1)

    def test_remember_stats_all_urls_looong_url(self):
        request = self.factory.get(
            "/something/really/long/to/start/with/right/here/since/this/will/"
            "test/that/things/work/with/long/urls/too",
            {
                "line1": "Bad luck, wind been blowing at my back",
                "line2": "I was born to bring trouble to wherever I'm at",
                "line3": "Got the number thirteen, tattooed on my neck",
                "line4": "When the ink starts to itch, ",
                "line5": "then the black will turn to red",
            },
        )
        response = views.home6(request)
        eq_(response.status_code, 200)

        # now ask the memory thing
        (match,) = find_urls()
        ok_(match[0].startswith("/something/really"))
        eq_(match[2]["hits"], 0)
        eq_(match[2]["misses"], 1)

        # second time
        response = views.home6(request)
        eq_(response.status_code, 200)
        (match,) = find_urls([])
        ok_(match[0].startswith("/something/really"))
        eq_(match[2]["hits"], 1)
        eq_(match[2]["misses"], 1)

    def test_cache_backends(self):
        request = self.factory.get("/anything")

        response = views.home7(request)
        eq_(response.status_code, 200)
        ok_(re.findall("Random:\w+", response.content.decode("utf8")))
        random_string_1 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]

        # clear second cache backend
        caches["second_backend"].clear()
        response = views.home7(request)
        eq_(response.status_code, 200)
        random_string_2 = re.findall(
            "Random:(\w+)", response.content.decode("utf8")
        )[0]
        ok_(random_string_1 != random_string_2)
